@@ -15,8 +15,14 @@ define([
     './com_Const', 
     './com_util', 
     './com_interface',
-    'text!vp_base/python/userCommand.py',
-], function(com_Const, com_util, com_interface, userCommandFile) {
+    __VP_TEXT_LOADER__('vp_base/python/userCommand.py'), // INTEGRATION: unified version of text loader
+    __VP_TEXT_LOADER__('vp_base/python/printCommand.py'), // INTEGRATION: unified version of text loader
+    __VP_TEXT_LOADER__('vp_base/python/fileNaviCommand.py'), // INTEGRATION: unified version of text loader
+    __VP_TEXT_LOADER__('vp_base/python/pandasCommand.py'), // INTEGRATION: unified version of text loader
+    __VP_TEXT_LOADER__('vp_base/python/variableCommand.py'), // INTEGRATION: unified version of text loader
+    __VP_TEXT_LOADER__('vp_base/python/visualizationCommand.py') // INTEGRATION: unified version of text loader
+], function(com_Const, com_util, com_interface, 
+            userCommandFile, printCommand, fileNaviCommand, pandasCommand, variableCommand, visualizationCommand) {
 	'use strict';
     //========================================================================
     // Define Inner Variable
@@ -39,7 +45,21 @@ define([
         //========================================================================
         // Constructor
         //========================================================================
-        constructor(initialData) {
+        /**
+         * 
+         * @param {*} initialData 
+         * @param {*} extensionType      extension type: notebook/colab/lab
+         */
+        constructor(extensionType='notebook', initialData={}) {
+            // initial mode
+            this.extensionType = extensionType;
+            this.parentSelector = 'body';
+            if (extensionType === 'notebook') {
+                this.parentSelector = '#site';
+            } else if (extensionType === 'colab' || extensionType === 'lab') {
+                // this.parentSelector = '.notebook-horizontal';
+                this.parentSelector = 'body';
+            }
             // initial configuration
             this.data = {
                 // Configuration
@@ -188,7 +208,8 @@ define([
                 vp_config_version: '1.0.0',
                 vp_signature: 'VisualPython',
                 vp_position: {},
-                vp_section_display: false,
+                // CHROME: default to display vp
+                vp_section_display: true,
                 vp_note_display: false,
                 vp_menu_width: Config.MENU_MIN_WIDTH,
                 vp_note_width: Config.BOARD_MIN_WIDTH
@@ -253,54 +274,183 @@ define([
          * - automatically restart on jupyter kernel restart (loadVisualpython.js)
          */
         readKernelFunction() {
-            var libraryList = [ 
-                'printCommand.py',
-                'fileNaviCommand.py',
-                'pandasCommand.py',
-                'variableCommand.py',
-                'visualizationCommand.py',
-                // 'userCommand.py'
-            ];
-            let promiseList = [];
-            libraryList.forEach(libName => {
-                var libPath = com_Const.PYTHON_PATH + libName
-                $.get(libPath).done(function(data) {
-                    var code_init = data;
-                    promiseList.push(vpKernel.execute(code_init));
-                }).fail(function() {
-                    console.log('visualpython - failed to read library file', libName);
+            // CHROME: change method to load py files ($.get -> require)
+            return new Promise(function(resolve, reject) {
+                var libraryList = [ 
+                    printCommand, fileNaviCommand, pandasCommand, variableCommand, visualizationCommand
+                ];
+                let promiseList = [];
+                // libraryList.forEach(libName => {
+                //     var libPath = com_Const.PYTHON_PATH + libName;
+                //     $.get(libPath).done(function(data) {
+                //         var code_init = data;
+                //         promiseList.push(vpKernel.execute(code_init, true));
+                //     }).fail(function() {
+                //         console.log('visualpython - failed to read library file', libName);
+                //     });
+                // });
+                libraryList.forEach(libCode => {
+                    promiseList.push(vpKernel.execute(libCode, true));
+                });
+                
+                // run all promises
+                let failed = false;
+                Promise.all(promiseList).then(function(resultObj) {
+                    ;
+                }).catch(function(resultObj) {
+                    failed = true;
+                    console.log('visualpython - failed to load library', resultObj);
+                    // TODO: show to restart kernel
+                }).finally(function() {
+                    if (!failed) {
+                        console.log('visualpython - loaded libraries', libraryList);
+                        resolve(true);
+                    } else {
+                        reject(false);
+                    }
                 });
             });
-            // run all promises
-            let failed = false;
-            Promise.all(promiseList).then(function(resultObj) {
-            }).catch(function(resultObj) {
-                failed = true;
-                console.log('visualpython - failed to load library', resultObj);
-            }).finally(function() {
-                if (!failed) {
-                    console.log('visualpython - loaded libraries', libraryList);
-                } else {
-                    console.log('visualpython - failed to load libraries');
-                }
-            });
+            
         }
 
         getMode() {
             return Config.serverMode;
         }
 
-        loadData(configKey = 'vpudf') {
+        _checkMounted() {
             return new Promise(function(resolve, reject) {
-                Jupyter.notebook.config.load();
-                Jupyter.notebook.config.loaded.then(function() {
-                    var data = Jupyter.notebook.config.data[configKey];
-                    if (data == undefined) {
-                        data = {};
+                try {
+                    vpKernel.getColabMounted().then(function(result) {
+                        if (result==='True') {
+                            resolve(true);
+                        } else {
+                            reject(false);
+                        }
+                    }).catch(function(err) {
+                        reject(false);
+                    })
+                } catch (ex) {
+                    reject(false);
+                }
+            });
+        }
+
+        /**
+         * CHROME: Read from colab
+         * @param {*} configKey config key to read
+         */
+        _readFromColab(configKey='vpudf') {
+            return new Promise(function(resolve, reject) {
+                // mounted
+                // read /content/drive/MyDrive/.visualpython
+                vpKernel.getColabConfig(configKey).then(function(resultObj) {
+                    let { result } = resultObj;
+                    try {
+                        if (result && result.trim() != '') {
+                            let parsedResult = JSON.parse(result);
+                            resolve(parsedResult);
+                        } else {
+                            resolve({});
+                        }
+                    } catch (err) {
+                        reject(err);
                     }
-                    resolve(data);
+                }).catch(function(err) {
+                    reject(err);
+                })
+            });
+        }
+
+        /**
+         * CHROME: Write to colab
+         * @param {*} data data to write
+         */
+        _writeToColab(data={}, configKey='vpudf') {
+            return new Promise(function(resolve, reject) {
+                // mounted
+                // write to /content/drive/MyDrive/.visualpython
+                vpKernel.setColabConfig(JSON.stringify(data), configKey).then(function(result) {
+                    resolve(result);
+                }).catch(function(err) {
+                    reject(err);
                 });
             });
+        }
+
+        /**
+         * LAB: Read from lab
+         * @param {*} configKey config key to read
+         */
+        _readFromLab(configKey='vpudf') {
+            return new Promise(function(resolve, reject) {
+                // mounted
+                // read USER_PATH/.visualpython
+                vpKernel.getLabConfig(configKey).then(function(resultObj) {
+                    let { result } = resultObj;
+                    try {
+                        if (result && result.trim() != '') {
+                            let parsedResult = JSON.parse(result);
+                            resolve(parsedResult);
+                        } else {
+                            resolve({});
+                        }
+                    } catch (err) {
+                        reject(err);
+                    }
+                }).catch(function(err) {
+                    reject(err);
+                })
+            });
+        }
+
+        /**
+         * LAB: Write to lab
+         * @param {*} data data to write
+         */
+        _writeToLab(data={}, configKey='vpudf') {
+            return new Promise(function(resolve, reject) {
+                // write to USER_PATH/.visualpython
+                vpKernel.setLabConfig(JSON.stringify(data), configKey).then(function(result) {
+                    resolve(result);
+                }).catch(function(err) {
+                    reject(err);
+                });
+            });
+        }
+
+        loadData(configKey = 'vpudf') {
+            let that = this;
+            return new Promise(function(resolve, reject) {
+                if (that.extensionType === 'notebook') {
+                    Jupyter.notebook.config.load();
+                    Jupyter.notebook.config.loaded.then(function() {
+                        var data = Jupyter.notebook.config.data[configKey];
+                        if (data == undefined) {
+                            data = {};
+                        }
+                        resolve(data);
+                    });
+                } else if (that.extensionType === 'colab') {
+                    // CHROME: edited to use .visualpython files
+                    that._checkMounted().then(function() {
+                        that._readFromColab('', configKey).then(function(result) {
+                            resolve(result);
+                        }).catch(function(err) {
+                            reject(err);
+                        })
+                    }).catch(function() {
+                        // not mounted
+                        reject('Colab Drive is not mounted!');
+                    })
+                } else if (that.extensionType === 'lab') {
+                    // CHROME: edited to use .visualpython files
+                    that._readFromLab('', configKey).then(function(result) {
+                        resolve(result);
+                    }).catch(function(err) {
+                        reject(err);
+                    })
+                }
+            });
         };
 
         /**
@@ -310,39 +460,92 @@ define([
          * @returns 
          */
         getData(dataKey='', configKey='vpudf') {
+            let that = this;
             return new Promise(function(resolve, reject) {
-                Jupyter.notebook.config.load();
-                Jupyter.notebook.config.loaded.then(function() {
-                    var data = Jupyter.notebook.config.data[configKey];
-                    if (data == undefined) {
-                        resolve(data);
-                        return;
-                    }
-                    if (dataKey == '') {
-                        resolve(data);
-                        return;
-                    }
-                    if (Object.keys(data).length > 0) {
-                        resolve(data[dataKey]);
-                        return;
-                    }
-                    reject('No data available.');
-                });
+                if (that.extensionType === 'notebook') {
+                    Jupyter.notebook.config.load();
+                    Jupyter.notebook.config.loaded.then(function() {
+                        var data = Jupyter.notebook.config.data[configKey];
+                        if (data == undefined) {
+                            resolve(data);
+                            return;
+                        }
+                        if (dataKey == '') {
+                            resolve(data);
+                            return;
+                        }
+                        if (Object.keys(data).length > 0) {
+                            resolve(data[dataKey]);
+                            return;
+                        }
+                        reject('No data available.');
+                    });
+                } else if (that.extensionType === 'colab') {
+                    // CHROME: use drive .visualpython files
+                    that._checkMounted().then(function() {
+                        that._readFromColab(configKey).then(function(result) {
+                            let data = result;
+                            if (data == undefined || data == {}) {
+                                resolve(data);
+                                return;
+                            }
+                            if (dataKey == '') {
+                                resolve(data);
+                                return;
+                            }
+                            if (Object.keys(data).length > 0) {
+                                resolve(data[dataKey]);
+                                return;
+                            }
+                            reject('No data available.');
+                        }).catch(function(err) {
+                            reject(err);
+                        })
+                    }).catch(function() {
+                        // not mounted
+                        reject('Colab Drive is not mounted!');
+                    })
+                } else if (that.extensionType === 'lab') {
+                    // LAB: use local .visualpython files
+                    that._readFromLab(configKey).then(function(result) {
+                        let data = result;
+                        if (data == undefined || data == {}) {
+                            resolve(data);
+                            return;
+                        }
+                        if (dataKey == '') {
+                            resolve(data);
+                            return;
+                        }
+                        if (Object.keys(data).length > 0) {
+                            resolve(data[dataKey]);
+                            return;
+                        }
+                        reject('No data available.');
+                    }).catch(function(err) {
+                        reject(err);
+                    })
+                }
             });
         }
 
         getDataSimple(dataKey='', configKey='vpudf') {
-            Jupyter.notebook.config.load();
-            var data = Jupyter.notebook.config.data[configKey];
-            if (data == undefined) {
+            if (this.extensionType === 'notebook') {
+                Jupyter.notebook.config.load();
+                var data = Jupyter.notebook.config.data[configKey];
+                if (data == undefined) {
+                    return undefined;
+                }
+                if (dataKey == '') {
+                    return data;
+                }
+                if (Object.keys(data).length > 0) {
+                    return data[dataKey];
+                }
+            } else if (this.extensionType === 'colab') {
+                // CHROME: TODO: no way to simply get data
                 return undefined;
             }
-            if (dataKey == '') {
-                return data;
-            }
-            if (Object.keys(data).length > 0) {
-                return data[dataKey];
-            }
             
             return undefined;
         }
@@ -353,16 +556,92 @@ define([
          * @param {String} configKey 
          */
         setData(dataObj, configKey='vpudf') {
-            // set data using key
-            Jupyter.notebook.config.loaded.then(function() {
-                Jupyter.notebook.config.update({[configKey]: dataObj});
+            let that = this;
+            return new Promise(function(resolve, reject) {
+                if (that.extensionType === 'notebook') {
+                    // set data using key
+                    Jupyter.notebook.config.loaded.then(function() {
+                        Jupyter.notebook.config.update({[configKey]: dataObj});
+                        resolve(true);
+                    });
+                } else if (that.extensionType === 'colab') {
+                    // CHROME: use .visualpython files
+                    that.getData('', configKey).then(function(data) {
+                        let newDataObj = {};
+                        if (data && typeof data === 'object') {
+                            newDataObj = {
+                                ...data
+                            };
+                        }
+                        newDataObj = {
+                            ...newDataObj,
+                            ...dataObj
+                        }
+                        that._writeToColab(newDataObj, configKey).then(function() {
+                            resolve();
+                        }).catch(function() {
+                            reject();
+                        });
+                    });
+                } else if (that.extensionType === 'lab') {
+                    // LAB: use .visualpython files
+                    that.getData('', configKey).then(function(data) {
+                        let newDataObj = {};
+                        if (data && typeof data === 'object') {
+                            newDataObj = {
+                                ...data
+                            };
+                        }
+                        newDataObj = {
+                            ...newDataObj,
+                            ...dataObj
+                        }
+                        that._writeToLab(newDataObj, configKey).then(function() {
+                            resolve();
+                        }).catch(function() {
+                            reject();
+                        });
+                    });
+                }
             });
         }
 
         removeData(key, configKey = 'vpudf') {
-             // if set value to null, it removes from config data
-            Jupyter.notebook.config.loaded.then(function() {
-                Jupyter.notebook.config.update({[configKey]: {[key]: null}});
+            let that = this;
+            return new Promise(function(resolve, reject) {
+                if (that.extensionType === 'notebook') {
+                    // if set value to null, it removes from config data
+                    Jupyter.notebook.config.loaded.then(function() {
+                        Jupyter.notebook.config.update({[configKey]: {[key]: null}});
+                    });
+                    resolve(true);
+                } else if (that.extensionType === 'colab') {
+                    // CHROME: use .visualpython files
+                    that.getData('', configKey).then(function(data) {
+                        let dataObj = data;
+                        delete dataObj[key];
+                        that._writeToColab(dataObj, configKey).then(function() {
+                            resolve(true);
+                        }).catch(function() {
+                            reject(false);
+                        });
+                    }).catch(function(err) {
+                        reject(false);
+                    })
+                } else if (that.extensionType === 'lab') {
+                    // LAB: use .visualpython files
+                    that.getData('', configKey).then(function(data) {
+                        let dataObj = data;
+                        delete dataObj[key];
+                        that._writeToLab(dataObj, configKey).then(function() {
+                            resolve(true);
+                        }).catch(function() {
+                            reject(false);
+                        });
+                    }).catch(function(err) {
+                        reject(false);
+                    })
+                }
             });
         }
 
@@ -372,19 +651,36 @@ define([
          * @param {String} configKey 
          */
         getMetadata(dataKey='', configKey='vp') {
-            let metadata = Jupyter.notebook.metadata[configKey];
-            if (metadata) {
-                // update this metadataSetting
-                this.metadataSettings = {
-                    ...this.metadataSettings,
-                    ...metadata
-                };
-                // no datakey, return all metadata
-                if (dataKey == '') {
-                    return metadata;
+            if (this.extensionType === 'notebook') {
+                let metadata = Jupyter.notebook.metadata[configKey];
+                if (metadata) {
+                    // update this metadataSetting
+                    this.metadataSettings = {
+                        ...this.metadataSettings,
+                        ...metadata
+                    };
+                    // no datakey, return all metadata
+                    if (dataKey == '') {
+                        return metadata;
+                    }
+                    return metadata[dataKey];
                 }
-                return metadata[dataKey];
-            }
+            } else if (this.extensionType === 'colab') {
+                // CHROME: use colab.global.notebookModel.metadata
+                let metadata = colab.global.notebookModel.metadata[configKey];
+                if (metadata) {
+                    // update this metadataSetting
+                    this.metadataSettings = {
+                        ...this.metadataSettings,
+                        ...metadata
+                    };
+                    // no datakey, return all metadata
+                    if (dataKey == '') {
+                        return metadata;
+                    }
+                    return metadata[dataKey];
+                }
+            } 
             return {};
         }
 
@@ -394,19 +690,28 @@ define([
          * @param {String} configKey 
          */
         setMetadata(dataObj, configKey='vp') {
-            let oldData = Jupyter.notebook.metadata[configKey];
-            Jupyter.notebook.metadata[configKey] = {
-                ...oldData,
-                ...dataObj
-            };
-            Jupyter.notebook.set_dirty();
+            if (this.extensionType === 'notebook') {
+                let oldData = Jupyter.notebook.metadata[configKey];
+                Jupyter.notebook.metadata[configKey] = {
+                    ...oldData,
+                    ...dataObj
+                };
+                Jupyter.notebook.set_dirty();
+
+            } else if (this.extensionType === 'colab') {
+                // CHROME: use colab.global.notebookModel.metadata
+                let oldData = colab.global.notebookModel.metadata[configKey];
+                colab.global.notebookModel.metadata[configKey] = {
+                    ...oldData,
+                    ...dataObj
+                };
+            }
 
             // update this metadataSetting
             this.metadataSettings = {
                 ...this.metadataSettings,
                 ...dataObj
             };
-
         }
 
         /**
@@ -414,7 +719,12 @@ define([
          * @param {String} configKey 
          */
         resetMetadata(configKey='vp') {
-            Jupyter.notebook.metadata[configKey] = {};
+            if (this.extensionType === 'notebook') {
+                Jupyter.notebook.metadata[configKey] = {};
+            } else if (this.extensionType === 'colab') {
+                // CHROME: use colab.global.notebookModel.metadata
+                colab.global.notebookModel.metadata[configKey] = {};
+            }
         }
 
         /**
@@ -472,7 +782,11 @@ define([
         checkVpVersion(background=false) {
             let that = this;
             let nowVersion = this.getVpInstalledVersion();
-            this.getPackageVersion().then(function(latestVersion) {
+            let packageName = 'visualpython';
+            if (this.extensionType === 'lab') {
+                packageName = 'jupyterlab-visualpython';
+            }
+            this.getPackageVersion(packageName).then(function(latestVersion) {
                 if (nowVersion === latestVersion) {
                     // if it's already up to date
                     // hide version update icon
@@ -507,15 +821,37 @@ define([
                                     break;
                                 case 1:
                                     // update
-                                    let info = [
-                                        '## Visual Python Upgrade',
-                                        'NOTE: ',
-                                        '- Refresh your web browser to start a new version.',
-                                        '- Save VP Note before refreshing the page.'
-                                    ];
-                                    com_interface.insertCell('markdown', info.join('\n'));
-                                    com_interface.insertCell('code', '!pip install visualpython --upgrade');
-                                    com_interface.insertCell('code', '!visualpy install');
+                                    if (that.extensionType === 'notebook') {
+                                        let info = [
+                                            '## Visual Python Upgrade',
+                                            'NOTE: ',
+                                            '- Refresh your web browser to start a new version.',
+                                            '- Save VP Note before refreshing the page.'
+                                        ];
+                                        com_interface.insertCell('markdown', info.join('\n'));
+                                        com_interface.insertCell('code', '!pip install visualpython --upgrade');
+                                        com_interface.insertCell('code', '!visualpy install');
+                                    } else if (that.extensionType === 'colab') {
+                                        // CHROME: update chrome extension
+                                        let info = [
+                                            '## Visual Python Upgrade',
+                                            'NOTE: ',
+                                            '- Go to chrome webstore and update visualpython',
+                                            '- Refresh your web browser to start a new version.',
+                                            '- Save VP Note before refreshing the page.'
+                                        ];
+                                        com_interface.insertCell('markdown', info.join('\n'));
+                                    } else if (that.extensionType === 'lab') {
+                                        // LAB: update lab extension
+                                        let info = [
+                                            '## Visual Python Upgrade',
+                                            'NOTE: ',
+                                            '- Refresh your web browser to start a new version.',
+                                            '- Save VP Note before refreshing the page.'
+                                        ];
+                                        com_interface.insertCell('markdown', info.join('\n'));
+                                        com_interface.insertCell('code', '!pip install jupyterlab-visualpython --upgrade');
+                                    }
 
                                     // update version_timestamp
                                     that.setData({ 'version_timestamp': new Date().getTime() }, 'vpcfg');
@@ -587,7 +923,7 @@ define([
     /**
      * Version
      */
-    Config.version = "2.2.12";
+    Config.version = "2.3.2";
 
     /**
      * Type of mode
