@@ -12,8 +12,9 @@ define([
     'vp_base/js/com/com_util',
     'vp_base/js/com/com_makeDom',
     'vp_base/js/com/component/SuggestInput',
-    'vp_base/js/com/component/VarSelector2'
-], function (com_util, com_makeDom, SuggestInput, VarSelector2) {
+    'vp_base/js/com/component/VarSelector2',
+    'vp_base/js/com/component/DataSelector'
+], function (com_util, com_makeDom, SuggestInput, VarSelector2, DataSelector) {
     /**
      * show result after code executed
      */
@@ -33,6 +34,7 @@ define([
         'bool_select': 'Select Boolean',
         'option_select': 'Select option',
         'option_suggest': 'Input option',
+        'data_select': 'Select data',
         'var_select': 'Select Variable',
         'var_multi': 'Select N-Variables',
         'col_select': 'Select Column',
@@ -43,6 +45,7 @@ define([
     }
 
     const _VP_BOOL_OPTIONS = [
+        { label: 'Select option...', value: ''},
         { label: 'True', value: 'True' },
         { label: 'False', value: 'False' }
     ]
@@ -118,13 +121,15 @@ define([
      * Generate page interface based on package configuration
      * @param {Component} pageThis
      * @param {*} package 
+     * @param {Object} state [optional]
+     * @param {String} parent [optional] parent selector as string
      */
-    var vp_showInterfaceOnPage = function(pageThis, package, state={}) {
+    var vp_showInterfaceOnPage = function(pageThis, package, state={}, parent='') {
 
         // generate input variable tag
-        var tblInput = $(pageThis.wrapSelector()+' #vp_inputOutputBox table tbody');
+        var tblInput = $(pageThis.wrapSelector() + (parent!==''?(' ' + parent):'') + ' #vp_inputOutputBox table tbody');
         // generate option variable tag
-        var tblOption = $(pageThis.wrapSelector()+' #vp_optionBox table tbody');
+        var tblOption = $(pageThis.wrapSelector() + (parent!==''?(' ' + parent):'') + ' #vp_optionBox table tbody');
 
         package.options && package.options.forEach(function(o, i) {
             var obj = JSON.parse(JSON.stringify(o));
@@ -257,6 +262,14 @@ define([
                 content = renderTabBlock(pageThis, obj, state);
                 break;
             case 'bool_checkbox':
+                content = $(`<input type="checkbox" id="${obj.name}" class="vp-checkbox"/>`);
+                if (value != undefined) {
+                    // set as saved value
+                    $(content).attr({
+                        'checked': value
+                    });
+                }
+                break;
             case 'bool_select':
                 // True False select box
                 var optSlct = $(`<select id="${obj.name}" class="vp-select vp-state"></select>`);
@@ -265,10 +278,11 @@ define([
                         // 'id':opt,
                         'index':obj.index,
                         'name':obj.name,
-                        'value':(obj.default==opt.value?'':opt.value)
+                        // 'value':(obj.default==opt.value?'':opt.value)
+                        'value':opt.value
                     });
                     // cell metadata test
-                    if (value != undefined) {
+                    if (value != undefined && obj.value != undefined) {
                         // set as saved value
                         if (value == opt.value) {
                             $(option).attr({
@@ -277,11 +291,11 @@ define([
                         }
                     } else {
                         // set default value
-                        if (value == opt.default) {
-                            $(option).attr({
-                                'selected':'selected'
-                            });
-                        }
+                        // if (value == opt.default) {
+                        //     $(option).attr({
+                        //         'selected':'selected'
+                        //     });
+                        // }
                     }
                     optSlct.append(option);
                 });
@@ -329,6 +343,9 @@ define([
                 if (obj.placeholder != undefined) {
                     suggestInput.setPlaceholder(obj.placeholder);
                 }
+                if (obj.required === true) {
+                    suggestInput.addAttribute('required', true);
+                }
                 suggestInput.setSelectEvent(function(selectedValue) {
                     // trigger change
                     $(pageThis.wrapSelector('#' + obj.name)).val(selectedValue);
@@ -336,12 +353,24 @@ define([
                 });
                 content = $(suggestInput.toTagString());
                 break;
+            case 'data_select':
+                let dataSelector = new DataSelector({
+                    pageThis: pageThis, 
+                    id: obj.name,
+                    allowDataType: obj.varType, 
+                    placeholder: obj.placeholder || 'Select data',
+                    value: value,
+                    required: obj.required === true
+                });
+                content = $(dataSelector.toTagString());
+                break;
             case 'var_select':
                 // suggest input tag
                 var tag = $('<input/>').attr({
-                    'type': 'text',
-                    'id': obj.name,
-                    'class': 'vp-input vp-state'
+                    type: 'text',
+                    id: obj.name,
+                    class: 'vp-input vp-state',
+                    required: obj.required === true
                 });
                 vp_generateVarSuggestInput(pageThis.wrapSelector(), obj);
                 content = tag;
@@ -354,7 +383,7 @@ define([
                     // multiple selection true
                     'multiple': true
                 });
-                vp_generateVarSelect(tag, obj.var_type, obj.value);
+                vp_generateVarSelect(tag, obj.varType, obj.value);
                 content = tag;
                 break;
             case 'col_select':
@@ -376,13 +405,23 @@ define([
                 break;
             case 'input_number':
                 var input = $('<input/>').attr({
-                    'type':'number',
-                    'class':'vp-input vp-state',
-                    'id':obj.name,
-                    'placeholder':(obj.placeholder==undefined?'Input Number':obj.placeholder),
-                    'value':(obj.default==undefined?'':obj.default),
-                    'title':(obj.help==undefined?'':obj.help)
+                    type: 'number',
+                    class: 'vp-input vp-state',
+                    id: obj.name,
+                    placeholder: (obj.placeholder==undefined?'Input Number':obj.placeholder),
+                    value: (obj.default==undefined?'':obj.default),
+                    title: (obj.help==undefined?'':obj.help),
+                    required: obj.required === true
                 });
+                if (obj.step != undefined) {
+                    $(input).attr({ 'step': obj.step });
+                }
+                if (obj.min != undefined) {
+                    $(input).attr({ 'min': obj.min });
+                }
+                if (obj.max != undefined) {
+                    $(input).attr({ 'max': obj.max });
+                }
                 // cell metadata test
                 if (value != undefined) {
                     // set as saved value
@@ -399,12 +438,13 @@ define([
             // default : input_single
             default:
                 var input = $('<input/>').attr({
-                    'type':'text',
-                    'class':'vp-input input-single vp-state',
-                    'id':obj.name,
-                    'placeholder':(obj.placeholder==undefined?'Input Data':obj.placeholder),
-                    'value':(obj.default==undefined?'':obj.default),
-                    'title':(obj.help==undefined?'':obj.help)
+                    type: 'text',
+                    class: 'vp-input input-single vp-state',
+                    id: obj.name,
+                    placeholder: (obj.placeholder==undefined?'Input Data':obj.placeholder),
+                    value: (obj.default==undefined?'':obj.default),
+                    title: (obj.help==undefined?'':obj.help),
+                    required: obj.required == true
                 });
                 // cell metadata test
                 if (value != undefined) {
@@ -425,7 +465,7 @@ define([
      * @param {object} obj
      */
     var vp_generateVarSuggestInput = function(divTag, obj) {
-        var types = obj.var_type;
+        var types = obj.varType;
         var defaultValue = obj.value;
         if (obj.value == undefined && obj.default != undefined) {
             defaultValue = obj.default;
@@ -450,7 +490,7 @@ define([
             let { result, type, msg } = resultObj;
             var varList = JSON.parse(result);
             varList = varList.map(function(v) {
-                return { label: v.varName + ' (' + v.varType + ')', value: v.varName, dtype: v.varType };
+                return { label: v.varName, value: v.varName, dtype: v.varType };
             });
             // 1. Target Variable
             var suggestInput = new SuggestInput();
@@ -459,6 +499,9 @@ define([
             suggestInput.setSuggestList(function() { return varList; });
             suggestInput.setNormalFilter(false);
             suggestInput.setValue(defaultValue);
+            if (obj.required === true) {
+                suggestInput.addAttribute('required', true);
+            }
             if (obj.placeholder != undefined) {
                 suggestInput.setPlaceholder(obj.placeholder);
             }
@@ -531,40 +574,46 @@ define([
 
     /**
      * Get tag value
-     * @param {Object} pageThis
-     * @param {*} obj 
-     * @returns {string} tag's value
+     * @param {Object} pageThis parent this object
+     * @param {Object} obj      state object to get tag value 
+     * @param {string} parent   parent selector as string
+     * @returns {string}        tag's value
      */
-    var vp_getTagValue = function(pageThis, obj) {
+    var vp_getTagValue = function(pageThis, obj, parent='') {
         var value = '';
         let componentType = 'input';
         if (obj.component && obj.component.length == 1) {
             componentType = obj.component[0];
         } else {
-            componentType = $(pageThis.wrapSelector('#' + obj.name + '_type')).val();
+            componentType = $(pageThis.wrapSelector(parent + ' #' + obj.name + '_type')).val();
         }
         switch (componentType) {
             case 'option_radio':
-                var input = $(pageThis.wrapSelector("input[name='"+obj.name+"']:checked")).val();
+                var input = $(pageThis.wrapSelector(parent + " input[name='"+obj.name+"']:checked")).val();
                 // same as default
                 if (input == obj.default) break;
                 value = input;
                 break;
             case 'option_checkbox':
-                var checked = $(pageThis.wrapSelector("input[name='"+obj.name+"']:checked")).val();
+                let checked = $(pageThis.wrapSelector(parent + " input[name='"+obj.name+"']:checked")).val();
 
                 for (var i = 0; i < checked.length; i++) {
                     value += "'" + $(checked[i]).val() + "',";
                 }
                 value = value.substr(0, value.length-1);
                 break;
+            case 'bool_checkbox':
+                let isChecked = $(pageThis.wrapSelector(parent + ' #'+obj.name)).prop('checked');
+                value = isChecked?'True':'False';
+                break;
             case 'input_multi':
             case 'bool_select':
+            case 'data_select':
             case 'var_select':
             case 'var_multi':
             case 'col_select':
             case 'dtype':
-                value = $(pageThis.wrapSelector('#'+obj.name)).val();
+                value = $(pageThis.wrapSelector(parent + ' #'+obj.name)).val();
                 break;
             case 'table':
             case 'file':
@@ -572,7 +621,7 @@ define([
             case 'option_suggest':
             case 'input_number':
             default:
-                var input = $(pageThis.wrapSelector('#'+obj.name)).val();
+                var input = $(pageThis.wrapSelector(parent + ' #'+obj.name)).val();
                 // same as default
                 if (input == obj.default) break;
                 value = input;
@@ -584,17 +633,19 @@ define([
      * Generate code
      * @param {Object} pageThis
      * @param {Object} package 
+     * @param {Object} state      [optional]
      * @param {string} etcOptions [optional] userOptionCode addition ex) ", test='TEST'"
+     * @param {string} parent     [optional] parent selector as string
      * @returns {string} generated code / if error, null
      */
-    var vp_codeGenerator = function(pageThis, package, state = {}, etcOptions = '') {
+    var vp_codeGenerator = function(pageThis, package, state = {}, etcOptions = '', parent='') {
         var code = package.code;
         
         try {
             package.options && package.options.forEach(function(v, i) {
                 var val = state[v.name];
                 if (val == undefined || val == '' || val == v.default) {
-                    val = vp_getTagValue(pageThis, v);
+                    val = vp_getTagValue(pageThis, v, parent=parent);
                 }
                 var id = '${' + v.name + '}';
                 if (val == undefined || val.trim() == '') {
@@ -641,8 +692,21 @@ define([
             // prevent code: no allocation variable ( = pd.DataFrame())
             if (code.startsWith(' = ')) {
                 code = code.substr(3);
+            } 
+            // show_result 
+            // get output variables
+            if (_VP_SHOW_RESULT && package.options) {
+                var outputOptList = package.options.filter(x => x.output === true);
+                var outputStr = '';
+                outputOptList.forEach(opt => {
+                    outputStr += (outputStr !== ''?', ':'') + vp_getTagValue(pageThis, opt);
+                })
+                if (outputStr != '') {
+                    code += '\n'+ outputStr;
+                }
             }
 
+
         } catch (e) {
             vpLog.display(VP_LOG_TYPE.ERROR, 'com_generator v2 code generation error ' + e.message);
             return null;
@@ -652,26 +716,24 @@ define([
 
     /**
      * Bind columns source function
-     * @param {string} selector thisWrapSelector 
-     * @param {object} target 
+     * @param {object} pageThis
+     * @param {object} targetId 
      * @param {array} columnInputIdList 
      * @param {string} tagType input / select (tag type)
      * @param {array/boolean} columnWithEmpty boolean array or value to decide whether select tag has empty space
+     * @param {array/boolean} columnWithIndex boolean array or value to decide whether select tag has index option
      * Usage : 
      *  $(document).on('change', this.wrapSelector('#dataframe_tag_id'), function() {
-     *      pdGen.vp_bindColumnSource(that.wrapSelector(), this, ['column_input_id'], 'select', [true, true, true]);
+     *      pdGen.vp_bindColumnSource(that, 'dataframe_tag_id', ['column_input_id'], 'select', [true, true, true]);
      *  });
      */
-    var vp_bindColumnSource = function(selector, target, columnInputIdList, tagType="input", columnWithEmpty=false) {
-        var varName = '';
-        if ($(target).length > 0) {
-            varName = $(target).val();
-        }
+    var vp_bindColumnSource = function(pageThis, targetId, columnInputIdList, tagType="input", columnWithEmpty=false, columnWithIndex=false) {
+        var varName = pageThis.state[targetId];
         if (varName === '') {
             // reset with no source
             columnInputIdList && columnInputIdList.forEach(columnInputId => {
-                let defaultValue = $(selector + ' #' + columnInputId).val();
-                if (defaultValue == null || defaultValue == undefined) {
+                let defaultValue = pageThis.state[columnInputId];
+                if (defaultValue === null || defaultValue === undefined) {
                     defaultValue = '';
                 }
                 if (tagType == 'input') {
@@ -702,6 +764,20 @@ define([
                 let { result, type, msg } = resultObj;
                 var varResult = JSON.parse(result);
 
+                // check if it needs to add index option
+                let addIndex = false;
+                if (Array.isArray(columnWithIndex)) {
+                    addIndex = columnWithIndex[idx];
+                } else {
+                    addIndex = columnWithIndex;
+                }
+                if (addIndex == true) {
+                    varResult = [
+                        {value: varName + '.index', label: 'index'},
+                        ...varResult
+                    ]
+                }
+
                 // check if it needs to add empty option
                 let addEmpty = false;
                 if (Array.isArray(columnWithEmpty)) {
@@ -711,15 +787,15 @@ define([
                 }
                 if (addEmpty == true) {
                     varResult = [
-                        {value: '', label: ''},
+                        {value: '', label: 'Select option...'},
                         ...varResult
                     ]
                 }
 
                 // columns using suggestInput
                 columnInputIdList && columnInputIdList.forEach((columnInputId, idx) => {
-                    let defaultValue = $(selector + ' #' + columnInputId).val();
-                    if (defaultValue == null || defaultValue == undefined) {
+                    let defaultValue = pageThis.state[columnInputId];
+                    if (defaultValue === null || defaultValue === undefined) {
                         defaultValue = '';
                     }
                     // create tag
@@ -748,16 +824,18 @@ define([
                                 'data-type':listVar.dtype
                             });
                             // cell metadata test : defaultValue as selected
-                            if (listVar.value == defaultValue) {
+                            if (listVar.value === defaultValue) {
                                 $(option).prop('selected', true);
                             }
                             option.append(document.createTextNode(listVar.label));
                             $(tag).append(option);
                         });
-                        $(selector + ' #' + columnInputId).replaceWith(function() {
+                        $(pageThis.wrapSelector('#' + columnInputId)).replaceWith(function() {
                             return $(tag);
                         });
                     }
+                }).catch(function(err) {
+                    vpLog.display(VP_LOG_TYPE.ERROR, 'com_generator - bindColumnSource error ', err)
                 });
                 
                 
