@@ -13,12 +13,14 @@
 // [CLASS] FileNavigation
 //============================================================================
 define([
-    'text!vp_base/html/component/fileNavigation.html!strip',
-    'css!vp_base/css/component/fileNavigation.css',
+    __VP_TEXT_LOADER__('vp_base/html/component/fileNavigation.html'), // INTEGRATION: unified version of text loader
+    __VP_CSS_LOADER__('vp_base/css/component/fileNavigation'), // INTEGRATION: unified version of css loader
+    'vp_base/js/com/component/LoadingSpinner',
+    'vp_base/js/com/com_Const',
     'vp_base/js/com/com_String',
     'vp_base/js/com/com_util',
     'vp_base/js/com/component/Component'
-], function(fileNaviHtml, fileNaviCss, com_String, com_util, Component) {
+], function(fileNaviHtml, fileNaviCss, LoadingSpinner, com_Const, com_String, com_util, Component) {
     // Temporary constant data
     const NAVIGATION_DIRECTION_TYPE = {
         TOP: 0,
@@ -47,7 +49,7 @@ define([
          * @param {Object} state { type, extensions, finish ... }
          */
         constructor(state) {
-            super($('#site'), state);
+            super($(vpConfig.parentSelector), state);
             /**
              * state.type           open / save
              * state.extensions     extensions list
@@ -84,6 +86,7 @@ define([
             this.currentFileList = [];
 
             this.pathState = {
+                parentPath: '',
                 currentPath: '',
                 baseFolder: '',     // root folder name
                 relativeDir: '',    // root folder name
@@ -120,9 +123,6 @@ define([
                     destDir: '/'
                 }
                 switch (pathType) {
-                    case '/':
-                        dirObj.direction = NAVIGATION_DIRECTION_TYPE.TOP;
-                        break;
                     case 'desktop':
                         dirObj.destDir = "_vp_get_desktop_path()";
                         dirObj.useFunction = true;
@@ -139,6 +139,13 @@ define([
                         dirObj.destDir = "_vp_get_userprofile_path()";
                         dirObj.useFunction = true;
                         break;
+                    case 'drive':
+                        dirObj.destDir = "/content/drive/MyDrive";
+                        break;
+                    case '/':
+                    default:
+                        dirObj.direction = NAVIGATION_DIRECTION_TYPE.TOP;
+                        break;
                 }
                 that.getFileList(dirObj);
             });
@@ -188,7 +195,16 @@ define([
 
         template() {
             /** Implement generating template */
-            return fileNaviHtml;
+            let fileNaviBody = $(fileNaviHtml.replaceAll('${vp_base}', com_Const.BASE_PATH));
+            $(fileNaviBody).find('.fnp-sidebar-menu').hide();
+            if (vpConfig.extensionType === 'notebook') {
+                $(fileNaviBody).find('.fnp-sidebar-menu.notebook').show();
+            } else if (vpConfig.extensionType === 'colab') {
+                $(fileNaviBody).find('.fnp-sidebar-menu.colab').show();
+            } else if (vpConfig.extensionType === 'lab') {
+                $(fileNaviBody).find('.fnp-sidebar-menu.lab').show();
+            }
+            return fileNaviBody;
         }
 
         /**
@@ -209,8 +225,7 @@ define([
 
             let directoryLi = $('<li class="fileNavigationPage-li"></li>');
             let directoryI = $(`<i class="fileNavigationPage-dir-text 
-                                            item_icon 
-                                            ${type=='dir'?'folder':'file'}_icon 
+                                            vp-${type=='dir'?'folder':'file'}-icon 
                                             icon-fixed-width"
                                     data-type="${type}"
                                     data-path="${path}"
@@ -257,9 +272,24 @@ define([
             let fileList = this.currentFileList;
             // clear body
             $(this.wrapSelector('.fileNavigationPage-body')).html('');
+
             // render file items
             let dirArr = [];
             let fileArr = [];
+
+            // render upper folder
+            if (that.pathState.parentPath != '') {
+                let upperFolderTag = that.renderFileItem({
+                    name: '..',
+                    type: 'dir',
+                    path: that.pathState.parentPath,
+                    size: '',
+                    atime: '',
+                    mtime: ''
+                });
+                dirArr.push(upperFolderTag);
+            }
+
             fileList && fileList.forEach((file, idx) => {
                 if (idx == 0) return;
                 let fileTag = that.renderFileItem(file);
@@ -295,7 +325,7 @@ define([
             /** render current path */
             var currentRelativePathStrArray = currentRelativePathStr.split('/');
             
-            if (baseFolder === notebookFolder && Jupyter.notebook.notebook_path.indexOf('/') === -1) {
+            if (baseFolder === notebookFolder && vpKernel.getNotebookPath().indexOf('/') === -1) {
                 var index = currentRelativePathStrArray.indexOf(baseFolder)
                 if (index >= 0) {
                     currentRelativePathStrArray.splice(index,1);
@@ -386,9 +416,7 @@ define([
             $(this.wrapSelector('#vp_fileNavigationInput')).on('change', function() {
                 let fileName = $(this).val();
                 let filePath = that.getRelativePath(that.pathState.baseDir, that.pathState.currentPath);
-                if (filePath == '') {
-                    filePath = './' + fileName;
-                }
+                
                 that.handleSelectFile(filePath, fileName);
             });
             // bind save cancel event
@@ -400,7 +428,7 @@ define([
                     let selectedExt = $(that.wrapSelector('#vp_fileNavigationExt')).val();
                     let fileExtIdx = fileName.lastIndexOf('.');
                     // if no extension, add it
-                    if (fileExtIdx < 0 || fileName.substring(fileExtIdx + 1) != selectedExt) {
+                    if (selectedExt != '' && (fileExtIdx < 0 || fileName.substring(fileExtIdx + 1) != selectedExt)) {
                         fileName += '.' + selectedExt;
                     }
                     // no path, set it
@@ -408,7 +436,7 @@ define([
                         filePath = './' + fileName;
                     }
                     fileExtIdx = filePath.lastIndexOf('.');
-                    if (fileExtIdx < 0 || filePath.substring(fileExtIdx + 1) != selectedExt) {
+                    if (selectedExt != '' && (fileExtIdx < 0 || filePath.substring(fileExtIdx + 1) != selectedExt)) {
                         filePath += '.' + selectedExt;
                     }
 
@@ -524,6 +552,8 @@ define([
             } else {
                 destDir = '.';
             }
+            // loading
+            let loadingSpinner = new LoadingSpinner($(this.wrapSelector('.fileNavigationPage-directory-container')));
             // get file list using kernel
             return vpKernel.getFileList(destDir, useFunction).then(function(result) {
                 /** Caution : if change code "$1" to '$1' as single quote, json parsing error occurs */
@@ -569,14 +599,36 @@ define([
                     });
                 }
 
+                vpLog.display(VP_LOG_TYPE.DEVELOP, 'FileNavigation - getFileList: ', filtered_varList);
 
                 var { currentDirStr,  currentRelativePathStr } = that.splitPathStrAndSetStack(dirObj, filtered_varList);
+                if (filtered_varList[0].current === filtered_varList[0].parent) {
+                    // no parent
+                    that.pathState.parentPath = '';
+                } else {
+                    that.pathState.parentPath = filtered_varList[0].parent; // parent path
+                }
                 that.pathState.relativeDir = currentRelativePathStr;
                 that.pathState.currentPath = currentDirStr;
                 that.currentFileList = filtered_varList;
 
-                that.loadFileList(); 
-            });;
+                that.loadFileList();
+            }).catch(function(resultObj) {
+                vpLog.display(VP_LOG_TYPE.ERROR, 'FileNavigation error', resultObj);
+
+                let { msg, result } = resultObj;
+                // show error using alert
+                
+                if (msg.content.evalue) {
+                    let resultStr = msg.content.evalue;
+                    //t.match(/\[Errno [0-9]+?\] (.*)/)[1]
+                    // get error message from traceback
+                    let alertMsg = resultStr.match(/\[Errno [0-9]+?\] (.*)/)[1];
+                    com_util.renderAlertModal(alertMsg);
+                }
+            }).finally(function() {
+                loadingSpinner.remove();
+            });
         }
 
         /**
@@ -605,7 +657,7 @@ define([
              *      RelativePathStr changes when moving through paths
              */
             var rootFolderName = splitedDirStrArr[splitedDirStrArr.length - 1];
-            var notebookPath = Jupyter.notebook.notebook_path;
+            var notebookPath = vpKernel.getNotebookPath();
 
             this.pathState.baseFolder = rootFolderName;
             this.pathState.relativeDir = rootFolderName;
