@@ -13,8 +13,8 @@
 // [CLASS] Instance
 //============================================================================
 define([
-    'text!vp_base/html/m_apps/instance.html!strip',
-    'css!vp_base/css/m_apps/instance.css',
+    __VP_TEXT_LOADER__('vp_base/html/m_apps/instance.html'), // INTEGRATION: unified version of text loader
+    __VP_CSS_LOADER__('vp_base/css/m_apps/instance'), // INTEGRATION: unified version of css loader
     'vp_base/js/com/com_String',
     'vp_base/js/com/component/PopupComponent',
     'vp_base/js/com/component/InstanceEditor',
@@ -32,9 +32,10 @@ define([
             /** Write codes executed before rendering */
             this.config.dataview = false;
             this.config.sizeLevel = 1;
+            this.config.checkModules = ['pd'];
 
             this.state = {
-                subsetEditor: undefined,
+                vp_instanceVariable: '',
                 variable: {
                     stack: []
                 },
@@ -43,6 +44,8 @@ define([
                 ...this.state
             }
             this.pointer = this.state.variable;
+            this.subsetEditor = null;
+            this.insEditor = null;
 
             this._addCodemirror('vp_instanceVariable', this.wrapSelector('#vp_instanceVariable'), 'readonly');
         }
@@ -149,51 +152,60 @@ define([
             });
 
             // co-op with Subset
-            $(this.wrapSelector('#vp_instanceVariable')).on('remove_option_page', function(evt) {
-                let component = evt.component;
-                component.close();
-            });
-            $(this.wrapSelector('#vp_instanceVariable')).on('close_option_page', function(evt) {
-                let component = evt.component;
-                component.close();
-            });
-            $(this.wrapSelector('#vp_instanceVariable')).on('focus_option_page', function(evt) {
-                let component = evt.component;
-                component.focus();
-            });
-            $(this.wrapSelector('#vp_instanceVariable')).on('apply_option_page', function(evt) {
-                let component = evt.component;
-                // apply its value
-                let code = component.generateCode();
-                component.close();
-                that.addStack();
-                that.state.subsetEditor.state.pandasObject = code;
-                that.updateValue(code);
-            });
+            // $(this.wrapSelector('#vp_instanceVariable')).on('remove_option_page', function(evt) {
+            //     let component = evt.component;
+            //     component.close();
+            // });
+            // $(this.wrapSelector('#vp_instanceVariable')).on('close_option_page', function(evt) {
+            //     let component = evt.component;
+            //     component.close();
+            // });
+            // $(this.wrapSelector('#vp_instanceVariable')).on('focus_option_page', function(evt) {
+            //     let component = evt.component;
+            //     component.focus();
+            // });
+            // $(this.wrapSelector('#vp_instanceVariable')).on('apply_option_page', function(evt) {
+            //     let component = evt.component;
+            //     // apply its value
+            //     let code = component.generateCode();
+            //     component.close();
+            //     that.addStack();
+            //     that.state.subsetEditor.state.pandasObject = code;
+            //     that.updateValue(code);
+            // });
         }
 
         templateForBody() {
-            return insHtml;
+            let page = $(insHtml);
+            $(page).find('#vp_instanceVariable').val(this.state.vp_instanceVariable);
+            return page;
         }
 
         render() {
             super.render();
 
+            let that = this;
+
             // vpSubsetEditor
-            this.state.subsetEditor = new Subset({ pandasObject: '', config: { name: 'Subset' } }, 
+            this.subsetEditor = new Subset({ pandasObject: '', config: { name: 'Subset' } }, 
                 { 
                     useInputVariable: true,
                     targetSelector: this.wrapSelector('#vp_instanceVariable'),
-                    pageThis: this
+                    pageThis: this,
+                    finish: function(code) {
+                        that.addStack();
+                        that.subsetEditor.state.pandasObject = code;
+                        that.updateValue(code);
+                    }
                 });
-            this.state.subsetEditor.disableButton();
+            this.subsetEditor.disableButton();
 
-            this.ALLOW_SUBSET_TYPES = this.state.subsetEditor.getAllowSubsetTypes();
+            this.ALLOW_SUBSET_TYPES = this.subsetEditor.getAllowSubsetTypes();
 
             // vpInstanceEditor
-            this.state.variable.insEditor = new InstanceEditor(this, "vp_instanceVariable", 'vp_variableInsEditContainer');
+            this.insEditor = new InstanceEditor(this, "vp_instanceVariable", 'vp_variableInsEditContainer');
 
-            this.state.variable.insEditor.show();
+            this.insEditor.show();
 
             // variable load
             this.reloadInsEditor();
@@ -225,17 +237,17 @@ define([
 
         hide() {
             super.hide();
-            this.state.subsetEditor && this.state.subsetEditor.hide();
+            this.subsetEditor && this.subsetEditor.hide();
         }
 
         close() {
             super.close();
-            this.state.subsetEditor && this.state.subsetEditor.close();
+            this.subsetEditor && this.subsetEditor.close();
         }
 
         remove() {
             super.remove();
-            this.state.subsetEditor && this.state.subsetEditor.remove();
+            this.subsetEditor && this.subsetEditor.remove();
         }
 
         updateValue(value) {
@@ -247,6 +259,7 @@ define([
                 cm.focus();
                 cm.setCursor({ line: 0, ch: value.length});
             }
+            this.state.vp_instanceVariable = value;
         }
 
         addStack() {
@@ -280,22 +293,17 @@ define([
                 var varType = varObj.type;
     
                 if (that.ALLOW_SUBSET_TYPES.includes(varType)) {
-                    that.state.subsetEditor.state.dataType = varType;
+                    that.subsetEditor.state.dataType = varType;
                     let cmObj = that.getCodemirror('vp_instanceVariable');
                     let nowCode = (cmObj && cmObj.cm)?cmObj.cm.getValue():'';
-                    that.state.subsetEditor.state.pandasObject = nowCode;
-                    that.state.subsetEditor.enableButton();
+                    that.subsetEditor.state.pandasObject = nowCode;
+                    that.subsetEditor.enableButton();
                 } else {
-                    that.state.subsetEditor.disableButton();
+                    that.subsetEditor.disableButton();
                 }
             };
     
-            if (type == '') {
-                this.pointer.insEditor.reload(callbackFunction);
-            } else {
-                tempPointer = this.state[type];
-                this.state[type].insEditor.reload(callbackFunction);
-            }
+            this.insEditor.reload(callbackFunction);
         }
     }
 
