@@ -13,8 +13,8 @@
 // [CLASS] Reshape
 //============================================================================
 define([
-    'text!vp_base/html/m_apps/reshape.html!strip',
-    'css!vp_base/css/m_apps/reshape.css',
+    __VP_TEXT_LOADER__('vp_base/html/m_apps/reshape.html'), // INTEGRATION: unified version of text loader
+    __VP_CSS_LOADER__('vp_base/css/m_apps/reshape'), // INTEGRATION: unified version of css loader
     'vp_base/js/com/com_String',
     'vp_base/js/com/com_util',
     'vp_base/js/com/component/PopupComponent',
@@ -30,6 +30,7 @@ define([
             super._init();
             /** Write codes executed before rendering */
             this.config.sizeLevel = 2;
+            this.config.checkModules = ['pd'];
 
             this.state = {
                 variable: '',
@@ -353,6 +354,7 @@ define([
             variableInput.setSuggestList(function () { return mappedList; });
             variableInput.setNormalFilter(true);
             variableInput.setValue(defaultValue);
+            variableInput.addAttribute('required', true);
             $(this.wrapSelector('#' + id)).replaceWith(function() {
                 return variableInput.toTagString();
             });
