@@ -13,15 +13,16 @@
 // [CLASS] Subset
 //============================================================================
 define([
-    'text!vp_base/html/m_apps/subset.html!strip',
-    'css!vp_base/css/m_apps/subset.css',
+    __VP_TEXT_LOADER__('vp_base/html/m_apps/subset.html'), // INTEGRATION: unified version of text loader
+    __VP_CSS_LOADER__('vp_base/css/m_apps/subset'), // INTEGRATION: unified version of css loader
+    'vp_base/js/com/com_Const',
     'vp_base/js/com/com_String',
     'vp_base/js/com/com_util',
     'vp_base/js/com/component/PopupComponent',
     'vp_base/js/com/component/SuggestInput',
     'vp_base/js/com/component/VarSelector',
-    'vp_base/js/com/component/MultiSelector'
-], function(subsetHtml, subsetCss, com_String, com_util, PopupComponent, SuggestInput, VarSelector, MultiSelector) {
+    'vp_base/js/com/component/DataSelector'
+], function(subsetHtml, subsetCss, com_Const, com_String, com_util, PopupComponent, SuggestInput, VarSelector, DataSelector) {
 
     /**
      * Subset
@@ -141,7 +142,18 @@ define([
         }
 
         templateForBody() {
-            return subsetHtml;
+            let page = $(subsetHtml);
+
+            let that = this;
+            let allocateSelector = new DataSelector({
+                pageThis: this, id: 'allocateTo', classes: VP_DS_ALLOCATE_TO, placeholder: 'New variable name',
+                finish: function() {
+                    that.generateCode();
+                }
+            });
+            $(page).find('.' + VP_DS_ALLOCATE_TO).replaceWith(allocateSelector.toTagString());
+
+            return page;
         }
         templateForDataView() {
             let tag = new com_String();
@@ -254,12 +266,21 @@ define([
 
             // row select - buttons
             tag.appendFormatLine('<div class="{0}">', VP_DS_SELECT_BTN_BOX);
-            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}">{3}</button>',
-                VP_DS_SELECT_ADD_ALL_BTN, 'select-row', 'Add all items', '<img src="/nbextensions/visualpython/img/arrow_right_double.svg"/></i>');
-            tag.appendFormatLine('<button type="button" class="{0} {1}">{2}</button>', VP_DS_SELECT_ADD_BTN, 'select-row', '<img src="/nbextensions/visualpython/img/arrow_right.svg"/>');
-            tag.appendFormatLine('<button type="button" class="{0} {1}">{2}</button>', VP_DS_SELECT_DEL_BTN, 'select-row', '<img src="/nbextensions/visualpython/img/arrow_left.svg"/>');
-            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}">{3}</button>',
-                VP_DS_SELECT_DEL_ALL_BTN, 'select-row', 'Remove all items', '<img src="/nbextensions/visualpython/img/arrow_left_double.svg"/>');
+            // LAB: img to url
+            // tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><img src="{3}"/></button>',
+            //     VP_DS_SELECT_ADD_ALL_BTN, 'select-row', 'Add all items', com_Const.IMAGE_PATH + 'arrow_right_double.svg');
+            // tag.appendFormatLine('<button type="button" class="{0} {1}"><img src="{2}"/></button>', VP_DS_SELECT_ADD_BTN, 'select-row', com_Const.IMAGE_PATH + 'arrow_right.svg');
+            // tag.appendFormatLine('<button type="button" class="{0} {1}"><img src="{2}"/></button>', VP_DS_SELECT_DEL_BTN, 'select-row', com_Const.IMAGE_PATH + 'arrow_left.svg');
+            // tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><img src="{3}"/></button>',
+            // VP_DS_SELECT_DEL_ALL_BTN, 'select-row', 'Remove all items', com_Const.IMAGE_PATH + 'arrow_left_double.svg');
+            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><div class="vp-icon-arrow-right-double"></div></button>',
+                VP_DS_SELECT_ADD_ALL_BTN, 'select-row', 'Add all items');
+            tag.appendFormatLine('<button type="button" class="{0} {1}"><div class="vp-icon-arrow-right"></div></button>',
+                VP_DS_SELECT_ADD_BTN, 'select-row');
+            tag.appendFormatLine('<button type="button" class="{0} {1}"><div class="vp-icon-arrow-left"></div></button>',
+                VP_DS_SELECT_DEL_BTN, 'select-row');
+            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><div class="vp-icon-arrow-left-double"></div></button>',
+                VP_DS_SELECT_DEL_ALL_BTN, 'select-row', 'Remove all items');
             tag.appendLine('</div>'); // VP_DS_SELECT_BTNS
 
             // row select - right
@@ -438,12 +459,21 @@ define([
 
             // col select - buttons
             tag.appendFormatLine('<div class="{0}">', VP_DS_SELECT_BTN_BOX);
-            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}">{3}</button>',
-                VP_DS_SELECT_ADD_ALL_BTN, 'select-col', 'Add all items', '<img src="/nbextensions/visualpython/img/arrow_right_double.svg"/></i>');
-            tag.appendFormatLine('<button type="button" class="{0} {1}">{2}</button>', VP_DS_SELECT_ADD_BTN, 'select-col', '<img src="/nbextensions/visualpython/img/arrow_right.svg"/></i>');
-            tag.appendFormatLine('<button type="button" class="{0} {1}">{2}</button>', VP_DS_SELECT_DEL_BTN, 'select-col', '<img src="/nbextensions/visualpython/img/arrow_left.svg"/>');
-            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}">{3}</button>',
-                VP_DS_SELECT_DEL_ALL_BTN, 'select-col', 'Remove all items', '<img src="/nbextensions/visualpython/img/arrow_left_double.svg"/>');
+            // LAB: img to url
+            // tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><img src="{3}"/></button>',
+            //     VP_DS_SELECT_ADD_ALL_BTN, 'select-col', 'Add all items', com_Const.IMAGE_PATH + 'arrow_right_double.svg');
+            // tag.appendFormatLine('<button type="button" class="{0} {1}"><img src="{2}"/></button>', VP_DS_SELECT_ADD_BTN, 'select-col', com_Const.IMAGE_PATH + 'arrow_right.svg');
+            // tag.appendFormatLine('<button type="button" class="{0} {1}"><img src="{2}"/></button>', VP_DS_SELECT_DEL_BTN, 'select-col', com_Const.IMAGE_PATH + 'arrow_left.svg');
+            // tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><img src="{3}"/></button>',
+            //     VP_DS_SELECT_DEL_ALL_BTN, 'select-col', 'Remove all items', com_Const.IMAGE_PATH + 'arrow_left_double.svg');
+            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><div class="vp-icon-arrow-right-double"></div></button>',
+                VP_DS_SELECT_ADD_ALL_BTN, 'select-col', 'Add all items');
+            tag.appendFormatLine('<button type="button" class="{0} {1}"><div class="vp-icon-arrow-right"></div></button>',
+                VP_DS_SELECT_ADD_BTN, 'select-col');
+            tag.appendFormatLine('<button type="button" class="{0} {1}"><div class="vp-icon-arrow-left"></div></button>',
+                VP_DS_SELECT_DEL_BTN, 'select-col');
+            tag.appendFormatLine('<button type="button" class="{0} {1}" title="{2}"><div class="vp-icon-arrow-left-double"></div></button>',
+                VP_DS_SELECT_DEL_ALL_BTN, 'select-col', 'Remove all items');
             tag.appendLine('</div>'); // VP_DS_SELECT_BTNS
 
             // col select - right
@@ -1100,7 +1130,7 @@ define([
                             that.beforeOpen(that);
                         }
                         that.open();
-                        $(that.wrapSelector()).css({ 'z-index': 205 }); // move forward
+                        $(that.wrapSelector()).css({ 'z-index': 1205 }); // move forward
                     }
                 });
 
