@@ -13,7 +13,7 @@
 // [CLASS] Clustering
 //============================================================================
 define([
-    'text!vp_base/html/m_ml/model.html!strip',
+    __VP_TEXT_LOADER__('vp_base/html/m_ml/model.html'), // INTEGRATION: unified version of text loader
     'vp_base/js/com/com_util',
     'vp_base/js/com/com_interface',
     'vp_base/js/com/com_String',
