@@ -13,8 +13,8 @@
 // [CLASS] ModelInfo
 //============================================================================
 define([
-    'text!vp_base/html/m_ml/modelInfo.html!strip',
-    'css!vp_base/css/m_ml/modelInfo.css',
+    __VP_TEXT_LOADER__('vp_base/html/m_ml/modelInfo.html'), // INTEGRATION: unified version of text loader
+    __VP_CSS_LOADER__('vp_base/css/m_ml/modelInfo'), // INTEGRATION: unified version of css loader
     'vp_base/js/com/com_util',
     'vp_base/js/com/com_String',
     'vp_base/js/com/com_generatorV2',
