@@ -13,8 +13,8 @@
 // [CLASS] Seaborn
 //============================================================================
 define([
-    'text!vp_base/html/m_visualize/seaborn.html!strip',
-    'css!vp_base/css/m_visualize/seaborn.css',
+    __VP_TEXT_LOADER__('vp_base/html/m_visualize/seaborn.html'), // INTEGRATION: unified version of text loader
+    __VP_CSS_LOADER__('vp_base/css/m_visualize/seaborn'), // INTEGRATION: unified version of css loader
     'vp_base/js/com/com_String',
     'vp_base/js/com/com_generatorV2',
     'vp_base/js/com/com_util',
@@ -309,7 +309,6 @@ define([
                 if (hue !== '') {
                     if (chartType == 'barplot' || chartType == 'countplot') {
                         let colDtype = $(that.wrapSelector('#hue')).find('option:selected').data('type');
-                        console.log(data, hue);
                         // get result and load column list
                         vpKernel.getColumnCategory(data, hue).then(function (resultObj) {
                             let { result } = resultObj;
@@ -807,7 +806,7 @@ define([
                     } else {
                         var errorContent = '';
                         if (msg.content.ename) {
-                            errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue);
+                            errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue, msg.content.detail);
                         }
                         $(that.wrapSelector('#chartPreview')).html(errorContent);
                         vpLog.display(VP_LOG_TYPE.ERROR, msg.content.ename, msg.content.evalue, msg.content);
@@ -816,7 +815,7 @@ define([
                     let { msg } = resultObj;
                     var errorContent = '';
                     if (msg.content.ename) {
-                        errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue);
+                        errorContent = com_util.templateForErrorBox(msg.content.ename, msg.content.evalue, msg.content.detail);
                     }
                     $(that.wrapSelector('#chartPreview')).html(errorContent);
                     vpLog.display(VP_LOG_TYPE.ERROR, msg.content.ename, msg.content.evalue, msg.content);
