@@ -13,22 +13,23 @@
 // [CLASS] MenuFrame
 //============================================================================
 define([
-    'text!../../html/menuFrame.html!strip',
-    'css!../../css/menuFrame.css',
+    __VP_TEXT_LOADER__('../../html/menuFrame.html'), // INTEGRATION: unified version of text loader
+    __VP_CSS_LOADER__('vp_base/css/menuFrame'), // INTEGRATION: unified version of css loader
 
     '../com/com_Config',
+    '../com/com_Const',
     '../com/com_util',
     '../com/com_interface',
     '../com/component/Component',
     '../com/component/SuggestInput',
     '../com/component/InnerFuncViewer',
 
-    'text!../../data/libraries.json',
+    __VP_RAW_LOADER__('../../data/libraries.json'), // INTEGRATION: text! to raw-loader
 
     './MenuGroup',
     './MenuItem',
     './TaskBar'
-], function(menuFrameHtml, menuFrameCss, com_Config, com_util, com_interface, Component, SuggestInput, InnerFuncViewer,
+], function(menuFrameHtml, menuFrameCss, com_Config, com_Const, com_util, com_interface, Component, SuggestInput, InnerFuncViewer,
             librariesJson, 
             MenuGroup, MenuItem, TaskBar) {
 	'use strict';
@@ -97,7 +98,12 @@ define([
                         break;
                     case 'restart':
                         // restart vp
-                        vpConfig.readKernelFunction();
+                        vpConfig.readKernelFunction().then(function() {
+                            // successfully restarted
+                            com_util.renderSuccessMessage('Successfully loaded inner functions for Visual Python');
+                        }).catch(function() {
+                            com_util.renderAlertModal('No connected runtime is detected. Please connect to runtime...');
+                        });
                         break;
                     case 'about':
                     case 'vpnote':
@@ -174,7 +180,13 @@ define([
          * @returns library object
          */
         getMenuLibraries() {
-            var libraries = JSON.parse(librariesJson);
+            var libraries = {};
+            // LAB: webpack5 load json object by default
+            if (vpConfig.extensionType === 'lab') {
+                libraries = librariesJson;
+            } else {
+                libraries = JSON.parse(librariesJson);
+            }
             if (!libraries || !libraries.library) {
                 vpLog.display(VP_LOG_TYPE.ERROR, 'vp menus are not avilable!');
                 return {};
@@ -206,7 +218,24 @@ define([
         }
         
         template() {
-            this.$pageDom = $(menuFrameHtml);
+            this.$pageDom = $(menuFrameHtml.replaceAll('${vp_base}', com_Const.BASE_PATH));
+            // LAB: vp_base replacing test
+            // const regexp = /"\$\{vp_base\}(.+?)"/g;
+            // var replaceHtml = menuFrameHtml;
+
+            // const rep_list = [...replaceHtml.matchAll(regexp)];
+
+            // for (var i=0; i<rep_list.length; i++) {
+            //     try {
+            //         var img_url = require(`..${rep_list[i][1]}`);
+            //         // var img_url = '/lib/visualpython' + rep_list[i][1];
+            //         replaceHtml = replaceHtml.replace(rep_list[i][0], `"${img_url}"`)
+            //     } catch (ex) {
+            //         ; // ignore error
+            //         console.log(ex);
+            //     }
+            // }
+            // this.$pageDom = $(replaceHtml);
             return this.$pageDom;
         }
 
@@ -252,7 +281,7 @@ define([
             menuLibraries && menuLibraries.forEach(item => {
                 if (item.type == 'package') {
                     // packages : MenuGroup
-                    var menuGroup = new MenuGroup($('#vp_menuBody'), item);
+                    var menuGroup = new MenuGroup($('.vp-menugroup-list'), item);
                     if (item.item) {
                         that.renderMenuItem(menuGroup);
                     }
